from json import loads

import requests
from lxml import etree
from lxml.builder import ElementMaker
from pylons import config
from pylons import request

import ckan.lib.base as base
import ckan.lib.helpers as h
import ckan.lib.navl.dictization_functions as dict_fns
import ckan.lib.render
import ckan.logic as logic
import ckan.model as model
import ckan.plugins.toolkit as toolkit
from ckan.common import _, c
from ckan.controllers.package import PackageController
from ckan.lib.mailer import mail_recipient
from ckan.logic import clean_dict
from ckan.logic import parse_params
from ckan.logic import tuplize_dict
from helpers import package_get_year

NotFound = logic.NotFound
NotAuthorized = logic.NotAuthorized
ValidationError = logic.ValidationError
abort = base.abort
get_action = logic.get_action

render = base.render
lookup_package_plugin = ckan.lib.plugins.lookup_package_plugin

doi_request_fields = [
    'Paper Title',
    'Conference Or Journal Title',
    'Author List',
    'DOI Title',
    'DOI Description',
    'Message to Admin (Optional)'
]

def build_xml(dataset):
    author = dataset['author']

    publisher = config['ckanext.ands.publisher']
    # Dev prefix default
    doi_prefix = config.get('ckanext.ands.doi_prefix', '10.5072/')

    # TODO what should this be?
    ands_client_id = config['ckanext.ands.client_id']

    namespaces = {
        'xsi': "http://www.w3.org/2001/XMLSchema-instance",
        None: "http://datacite.org/schema/kernel-3",
    }

    Root = ElementMaker(
        nsmap=namespaces
    )
    E = ElementMaker()

    xml = Root.resource(
        E.identifier(doi_prefix + ands_client_id, identifierType="DOI"),
        E.creators(E.creator(E.creatorName(author))),
        E.titles(E.title(dataset['title'])),
        E.publisher(publisher),
        E.publicationYear("{}".format(package_get_year(dataset))),
        E.language('en'),
        E.resourceType('gDMCP Dataset', resourceTypeGeneral="Dataset"),
        E.descriptions(E.description(dataset['notes'], descriptionType="Abstract")),
    )

    xml.attrib[etree.QName(namespaces['xsi'],
                           'schemaLocation')] = "http://datacite.org/schema/kernel-3 http://schema.datacite.org/meta/kernel-3/metadata.xsd"

    return etree.tostring(xml)


def post_doi_request(dataset_url, contents):
    app_id = config['ckanext.ands.DOI_API_KEY']
    shared_secret = config['ckanext.ands.shared_secret']

    mint_service_url = (
        'https://services.ands.org.au/doi/1.1/mint.json/?app_id={}&url={}&debug={}'.format(
            app_id, dataset_url, config.get('ckanext.ands.debug', False)))

    #  Send data
    return requests.post(mint_service_url, data={'xml': contents, 'shared_secret': shared_secret})


class DatasetDoiController(PackageController):
    def dataset_doi(self, id):
        if request.method == 'POST':
            return self.handle_submit(id)
        else:
            return self.doi_form(id)

    def dataset_doi_admin(self, id):
        xml_url = dataset_url = toolkit.url_for(
            controller='package',
            action='read',
            id=id,
            qualified=True
        )

        if request.method != 'POST' or not c.userobj.sysadmin:
            return toolkit.redirect_to(dataset_url)

        dataset = toolkit.get_action('package_show')(None, {'id': id})

        # If running on local machine, just resolve DOI to the dev server
        if 'localhost' in dataset_url or '127.0.0.1' in dataset_url:
            xml_url = config.get('ckanext.ands.debug_url')

        post_data = build_xml(dataset)
        resp = post_doi_request(xml_url, post_data)
        try:
            json = loads(resp.content)
        except ValueError as exp:
            h.flash_error("Invalid response from DOI server: {} :: {}".format(exp, resp.content))
            return toolkit.redirect_to(dataset_url)
        try:
            response_dict = json["response"]
        except KeyError:
            h.flash_error("Response had no response key: {}".format(json))
            return toolkit.redirect_to(dataset_url)

        doi = response_dict["doi"]
        response_code = response_dict["responsecode"]
        success_code = "MT001"

        if response_code == success_code:
            dataset['doi_id'] = doi
            toolkit.get_action('package_update')(None, dataset)
            h.flash_success("DOI Created successfully")
        else:
            type = response_dict["type"]
            err_msg = response_dict["message"]
            verbose_msg = response_dict["verbosemessage"]
            msg = response_code + ' - ' + type + '. ' + verbose_msg + '. ' + err_msg
            h.flash_error(_(msg))

        return toolkit.redirect_to(dataset_url)

    def doi_form(self, id):
        # Mostly copied from PackageController:read
        context = {'model': model, 'session': model.Session,
                   'user': c.user or c.author, 'for_view': True,
                   'auth_user_obj': c.userobj}

        data_dict = {'id': id, 'include_tracking': True}

        # interpret @<revision_id> or @<date> suffix

        # check if package exists
        try:
            c.pkg_dict = get_action('package_show')(context, data_dict)
            c.pkg = context['package']
        except NotFound:
            abort(404, _('Dataset not found'))
        except NotAuthorized:
            abort(401, _('Unauthorized to read package %s') % id)

        # used by disqus plugin
        c.current_package_id = c.pkg.id
        c.related_count = c.pkg.related_count

        package_type = c.pkg_dict['type'] or 'dataset'
        self._setup_template_variables(context, {'id': id},
                                       package_type=package_type)

        ########################## ADD DOI STUFF ###########################
        template = 'package/doi.html'
        fields = dict(
            (field.lower().replace(' ', '_'), field)
            for field in doi_request_fields
        )
        try:
            return render(template,
                          extra_vars={
                              'dataset_type': package_type,
                              # TODO
                              'data': {},
                              'errors': None,
                              'fields': fields
                          })
        except ckan.lib.render.TemplateNotFound:
            msg = _("Viewing {package_type} datasets in {format} format is "
                    "not supported (template file {file} not found).".format(
                package_type=package_type, format=format,
                file=template))
            abort(404, msg)

    def handle_submit(self, id):
        data = clean_dict(dict_fns.unflatten(tuplize_dict(parse_params(
            request.params))))

        data['dataset_url'] = toolkit.url_for(
            controller='package',
            action='read',
            id=id,
            qualified=True
        )

        # Comma separated config var
        to_addrs = config['ckanext.ands.support_emails'].split(',')

        subject = 'DataPortal Support: Request to publish dataset'

        body = base.render(
            'package/doi_email.text',
            extra_vars=data)

        for email in to_addrs:
            mail_recipient('Dataportal support', email, subject, body)

        h.flash_success("DOI Request sent")
        return toolkit.redirect_to(data['dataset_url'])


import ckan.plugins as plugins
import ckan.plugins.toolkit as toolkit
import helpers as h


class AndsPlugin(plugins.SingletonPlugin, toolkit.DefaultDatasetForm):
    plugins.implements(plugins.IConfigurer)
    plugins.implements(plugins.IRoutes, inherit=True)
    plugins.implements(plugins.IDatasetForm, inherit=True)
    plugins.implements(plugins.IPackageController, inherit=True)
    plugins.implements(plugins.ITemplateHelpers, inherit=True)

    # IConfigurer

    def update_config(self, config_):
        toolkit.add_template_directory(config_, 'templates')
        toolkit.add_public_directory(config_, 'public')
        toolkit.add_resource('fanstatic', 'ands')

    # IRoutes
    def after_map(self, map):
        map.connect(
            '/dataset/{id}/doi', action='dataset_doi',
            controller='ckanext.ands.controller:DatasetDoiController')
        map.connect(
            '/dataset/{id}/doi_approve', action='dataset_doi_admin',
            controller='ckanext.ands.controller:DatasetDoiController')
        return map

    # IDatasetForm
    def _modify_package_schema(self, schema):
        schema.update({
            'author': [toolkit.get_validator('not_empty'), unicode],
            'doi_id': [
                toolkit.get_converter('ignore_missing'),
                toolkit.get_converter('ignore_not_sysadmin'),
                toolkit.get_converter('convert_to_extras'),
            ]
        })
        return schema

    def create_package_schema(self):
        schema = super(AndsPlugin, self).create_package_schema()
        return self._modify_package_schema(schema)

    def update_package_schema(self):
        schema = super(AndsPlugin, self).update_package_schema()
        return self._modify_package_schema(schema)

    def show_package_schema(self):
        schema = super(AndsPlugin, self).show_package_schema()
        schema.update({
            'doi_id': [
                toolkit.get_converter('convert_from_extras'),
                toolkit.get_validator('ignore_missing')]
        })
        return schema

    def is_fallback(self):
        # Return True to register this plugin as the default handler for
        # package types not handled by any other IDatasetForm plugin.
        return True

    def package_types(self):
        # This plugin doesn't handle any special package types, it just
        # registers itself as the default (above).
        return []

    # IPackageController
    def edit(self, entity):
        # DOI key shouldn't be deleted!
        for extra in entity.extras_list:
            if extra.key == 'doi_id' and extra.value is not None:
                extra.state = 'active'

    # ITemplateHelpers
    def get_helpers(self):
        return {
            'package_get_year': h.package_get_year,
            'now': h.now,
            'get_site_title': h.get_site_title
        }



"""Tests for plugin.py."""
import json
from datetime import datetime

import ckan
import requests
from ckan import model
from ckan.lib.helpers import url_for
from ckan.tests import factories
from ckan.tests import helpers
from ckan.tests.helpers import FunctionalTestBase, _get_test_app
from mock import Mock
from mock import call
from mock import patch
from nose.tools import assert_equal
from pylons import config

import ckanext.ands.controller
from ckanext.ands.controller import build_xml, post_doi_request, doi_request_fields

test_dataset_dict = {
    'author': 'An Author',
    'title': 'A dataset to do things with',
    'metadata_created': datetime.now(),
    'notes': 'Some notes about this dataset',
}


class TestAndsController(FunctionalTestBase):
    def __init__(self):
        self.app = None

    @classmethod
    def setup_class(cls):
        super(cls, cls).setup_class()
        helpers.reset_db()

    def teardown(self):
        ckan.plugins.unload('ands')

    def setup(self):
        self.app = _get_test_app()
        ckan.plugins.load('ands')

    def test_build_xml(self):
        # Mainly just check it works - no point checking lxml is working.
        build_xml(test_dataset_dict)

    def test_post_doi_request(self):
        test_dataset_url = 'http://blah.com'
        contents = ''
        with patch.object(requests, 'post') as mock_post:
            post_doi_request(test_dataset_url, contents)

        expected = [
            call('https://services.ands.org.au/doi/1.1/mint.json/?app_id=atestdoikey&url={}&debug=False'.format(
                test_dataset_url),
                data={'xml': contents, 'shared_secret': 'atestdoisecret'})]

        assert_equal(mock_post.mock_calls, expected)

    def test_dataset_has_doi_request_non_sysadmin(self):
        model.repo.rebuild_db()
        dataset = factories.Dataset(author='test author')
        response = self.app.get(url_for(
            controller='package', action='read',
            id=dataset['name']))
        response.mustcontain('Request DOI')
        response.mustcontain(no='Approve DOI')
        response.mustcontain(no='Cite this as')

    def test_dataset_has_doi_approve_sysadmin(self):
        model.repo.rebuild_db()
        dataset = factories.Dataset(author='test author')
        sysadmin = factories.Sysadmin()
        env = {'REMOTE_USER': sysadmin['name'].encode('ascii')}
        response = self.app.get(url_for(
            controller='package', action='read',
            id=dataset['name']), extra_environ=env)

        response.mustcontain('Request DOI')
        response.mustcontain('Approve DOI')
        response.mustcontain(no='Cite this as')

    def test_dataset_doi_admin_sysadmin(self):
        model.repo.rebuild_db()
        dataset = factories.Dataset(author='test author')
        sysadmin = factories.Sysadmin()
        env = {'REMOTE_USER': sysadmin['name'].encode('ascii')}
        url = url_for(
            controller='ckanext.ands.controller:DatasetDoiController', action='dataset_doi_admin',
            id=dataset['name'])
        mock_response = Mock(content=json.dumps(dict(response=dict(responsecode='MT001', doi='testdoi'))))
        mock_post = Mock(return_value=mock_response)
        with patch.object(requests, 'post', new=mock_post):
            response = self.app.post(url, extra_environ=env)

        # Don't bother checking the mocks, other tests do this

        response = response.follow(extra_environ=env)
        # Shouldn't appear as already created
        response.mustcontain(no='Approve DOI')
        response.mustcontain(no='Request DOI')
        response.mustcontain('Cite this as')

    def test_dataset_doi_admin_non_sysadmin(self):
        model.repo.rebuild_db()
        dataset = factories.Dataset(author='test author')
        user = factories.User()
        env = {'REMOTE_USER': user['name'].encode('ascii')}
        # Should redirect back to dataset page
        response = self.app.post(url_for(
            controller='ckanext.ands.controller:DatasetDoiController', action='dataset_doi_admin',
            id=dataset['name']), extra_environ=env)

        response = response.follow(extra_environ=env)
        response.mustcontain(no='Approve DOI')
        response.mustcontain(no='Cite this as')
        response.mustcontain('Request DOI')

    def test_dataset_doi_request(self):
        model.repo.rebuild_db()
        dataset = factories.Dataset(author='test author')
        user = factories.User()
        env = {'REMOTE_USER': user['name'].encode('ascii')}
        # Should redirect back to dataset page
        response = self.app.get(url_for(
            controller='ckanext.ands.controller:DatasetDoiController', action='dataset_doi',
            id=dataset['name']), extra_environ=env)
        for field in doi_request_fields:
            response.mustcontain(field)

        form = response.forms['dataset-doi']
        for field in form.fields:
            if field != 'save':
                form.set(field, 'test')

        with patch.object(ckanext.ands.controller, 'mail_recipient') as mock_mail:
            response = form.submit('submit', extra_environ=env)

        assert_equal(mock_mail.mock_calls, [call(
            'Dataportal support',
            config.get('ckanext.ands.support_emails'),
            'DataPortal Support: Request to publish dataset',
            u'A DOI has been requested\n\nDataset:  http://test.ckan.net/dataset/test_dataset_2\n\n'
            u'Paper Title:  test\nConference Title: \nAuthor List: test\nDOI Title: test\n'
            u'DOI Description: test\nOptional Message: ')])



#!/usr/bin/env python
# encoding: utf-8
"""
Created by 'bens3' on 2013-06-21.
Copyright (c) 2013 'bens3'. All rights reserved.
"""

from pylons import config
from datetime import datetime
import dateutil.parser as parser

def package_get_year(pkg_dict):
    """
    Helper function to return the package year published
    @param pkg_dict:
    @return:
    """
    if not isinstance(pkg_dict['metadata_created'], datetime):
        pkg_dict['metadata_created'] = parser.parse(pkg_dict['metadata_created'])

    return pkg_dict['metadata_created'].year

def get_site_title():
    """
    Helper function to return the config site title, if it exists
    @return: str site title
    """
    return config.get("ckanext.doi.site_title")

def now():
    return datetime.now()
